<?php

namespace Drupal\blocktheme\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Theme\Registry;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Create form for admin page.
 */
class BlockthemeAdminSettingsForm extends ConfigFormBase {

  /**
   * Returns the theme.registry service.
   *
   * @var \Drupal\Core\Theme\Registry
   */
  protected $themeRegistry;

  /**
   * Constructs a BlockthemeAdminSettingsForm form.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Theme\Registry $theme_registry
   *   Defines the theme registry service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, Registry $theme_registry) {
    parent::__construct($config_factory);

    $this->themeRegistry = $theme_registry;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('theme.registry')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'blocktheme.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'blocktheme_admin_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('blocktheme.settings');

    $form['help'] = [
      '#markup' => t('BlockTheme allows an admin to define twig files for standard block templates and provides a select box on the block configure form so the user can select a twig file to use as opposed to having to override the templates by block ID.'),
    ];

    $form['templates'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Custom Block Templates'),
      '#description' => $this->t('Enter one value per row in the form: <em>customtemplate|Friendly Name</em>, where "customtemplate" corresponds to a twig file called <em>block--blocktheme--customtemplate.html.twig</em> as well as to the value of an extra variable <em>blocktheme</em> in the block template.'),
      '#default_value' => $config->get('templates'),
    ];

    $form['show'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show Custom Block Theme'),
      '#description' => $this->t('Show the custom block theme used for a block in the %block_admin_page.', [
        '%block_admin_page' => Link::fromTextAndUrl('block admin page', Url::fromRoute('block.admin_display'))->toString(),
      ]),
      '#default_value' => $config->get('show'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('blocktheme.settings')
      ->set('templates', $form_state->getValue('templates'))
      ->set('show', (bool) $form_state->getValue('show'))
      ->save();

    parent::submitForm($form, $form_state);
    $this->themeRegistry->reset();
  }

}
